@@ -207,6 +207,7 @@ class HooksInstaller:
             return {"removed": [], "total": 0}
 
         removed = []
+        empty_types = []
 
         for hook_type, hooks in settings["hooks"].items():
             # Filter out WatchCode hooks
@@ -219,9 +220,13 @@ class HooksInstaller:
                 settings["hooks"][hook_type] = filtered_hooks
                 removed.append(hook_type)
 
-            # Clean up empty hook types
+            # Track empty hook types for cleanup after iteration
             if not filtered_hooks:
-                del settings["hooks"][hook_type]
+                empty_types.append(hook_type)
+
+        # Clean up empty hook types (after iteration to avoid dict size change)
+        for hook_type in empty_types:
+            del settings["hooks"][hook_type]
 
         self.save_settings(settings)
 
