@@ -23,8 +23,62 @@ import subprocess
 import logging
 import time
 import uuid
+import atexit
 from pathlib import Path
 from datetime import datetime
+from typing import Optional
+
+# LATENCY OPTIMIZATION: Use httpx with HTTP/2 for relay polling
+try:
+    import httpx
+    HTTPX_AVAILABLE = True
+except ImportError:
+    HTTPX_AVAILABLE = False
+
+# E2E encryption support
+try:
+    from watchcode.crypto import CryptoManager, is_encrypted
+    CRYPTO_AVAILABLE = True
+    _crypto_manager: Optional[CryptoManager] = None
+
+    def _get_crypto_manager() -> CryptoManager:
+        """Get or create crypto manager instance."""
+        global _crypto_manager
+        if _crypto_manager is None:
+            _crypto_manager = CryptoManager()
+        return _crypto_manager
+except ImportError:
+    CRYPTO_AVAILABLE = False
+    is_encrypted = lambda x: False  # noqa: E731
+    def _get_crypto_manager():
+        return None
+
+# Module-level HTTP client for connection reuse
+_relay_client: Optional['httpx.Client'] = None
+
+
+def _get_relay_client() -> 'httpx.Client':
+    """Get or create shared HTTP client for relay polling."""
+    global _relay_client
+    if _relay_client is None and HTTPX_AVAILABLE:
+        _relay_client = httpx.Client(
+            timeout=5.0,
+            http2=True,
+            limits=httpx.Limits(keepalive_expiry=60.0)
+        )
+        atexit.register(_cleanup_relay_client)
+    return _relay_client
+
+
+def _cleanup_relay_client():
+    """Clean up HTTP client on exit."""
+    global _relay_client
+    if _relay_client is not None:
+        try:
+            _relay_client.close()
+        except Exception:
+            pass
+        _relay_client = None
 
 # Configure logging
 logging.basicConfig(
@@ -38,13 +92,10 @@ logger = logging.getLogger(__name__)
 MAX_INPUT_SIZE = 1024 * 1024  # 1MB
 ALLOWED_HOOKS = {'stop', 'notification', 'notification_debug', 'session_start', 'session_end', 'subagent_stop', 'permission_request', 'pre_tool_use'}
 APNS_SENDER = Path.home() / ".watchcode" / "apns_sender.py"
-RESPONSE_LISTENER = Path.home() / ".watchcode" / "response_listener.py"
-CLOUDKIT_POLLER = Path.home() / ".watchcode" / "cloudkit_poller.py"
 DEBUG_LOG = Path.home() / ".watchcode" / "hook_debug.log"
-ICLOUD_DRIVE_PATH = Path.home() / "Library" / "Mobile Documents" / "iCloud~com~watchcode~app" / "Documents"
+# NOTE: CloudKit/iCloud paths removed 2026-01-02 (security: relay-only now)
 
-# Wake push configuration
-WAKE_PUSH_INTERVAL = 8  # seconds between wake pushes
+# Response polling configuration
 DEFAULT_PERMISSION_TIMEOUT = 55  # seconds to wait for Watch response
 
 # Relay server configuration
@@ -872,13 +923,11 @@ def handle_notification(data: dict) -> int:
         log_debug(f"[seq={sequence_id}] Failed to send notification")
         return 0  # Don't block, let terminal handle it
 
-    # Wait for Watch response
+    # Wait for Watch response via relay server
     log_debug(f"[seq={sequence_id}] Waiting for Watch response...")
-    response = wait_for_response_with_wake(
-        session_id=session_id,
+    response = wait_for_response(
         sequence_id=sequence_id,
         timeout=DEFAULT_PERMISSION_TIMEOUT,
-        wake_interval=WAKE_PUSH_INTERVAL,
         notification_id=notification_id
     )
 
@@ -951,99 +1000,11 @@ def handle_subagent_stop(data: dict) -> int:
     )
 
 
-def wait_for_response(session_id: str, timeout: int = 55) -> dict | None:
-    """Run response_listener.py and return parsed response.
-
-    Returns None if timeout or error, dict with 'action' key if successful.
-    """
-    if not RESPONSE_LISTENER.exists():
-        log_debug(f"response_listener.py not found at {RESPONSE_LISTENER}")
-        return None
-
-    cmd = [
-        sys.executable,
-        str(RESPONSE_LISTENER),
-        '--session-id', session_id,
-        '--timeout', str(timeout)
-    ]
-
-    try:
-        # Build safe environment
-        safe_env = os.environ.copy()
-        for var in ['LD_PRELOAD', 'LD_LIBRARY_PATH', 'DYLD_INSERT_LIBRARIES',
-                    'BASH_ENV', 'CDPATH', 'ENV']:
-            safe_env.pop(var, None)
-        safe_env['PATH'] = '/opt/homebrew/bin:/usr/local/bin:/usr/bin:/bin'
-
-        result = subprocess.run(
-            cmd,
-            capture_output=True,
-            text=True,
-            timeout=timeout + 5,  # Extra buffer for subprocess overhead
-            env=safe_env
-        )
-
-        log_debug(f"response_listener exit code: {result.returncode}")
-        if result.stderr:
-            log_debug(f"response_listener stderr: {result.stderr[:200]}")
-
-        if result.returncode == 0 and result.stdout.strip():
-            try:
-                return json.loads(result.stdout)
-            except json.JSONDecodeError as e:
-                log_debug(f"Failed to parse response JSON: {e}")
-                return None
-        else:
-            log_debug(f"No valid response (exit code {result.returncode})")
-            return None
-
-    except subprocess.TimeoutExpired:
-        log_debug(f"response_listener timed out after {timeout}s")
-        return None
-    except Exception as e:
-        log_debug(f"wait_for_response error: {e}")
-        return None
-
-
 def generate_sequence_id() -> int:
     """Generate a unique sequence ID for correlation tracking."""
     return int(time.time() * 1000) % 1000000  # 6-digit millisecond-based ID
 
 
-def check_icloud_response(session_id: str) -> dict | None:
-    """Direct file check for iCloud Drive response (faster than subprocess).
-
-    Checks: ~/Library/Mobile Documents/iCloud~com~watchcode~app/Documents/response_{session_id}.json
-
-    Returns response dict if found and valid, None otherwise.
-    """
-    response_file = ICLOUD_DRIVE_PATH / f"response_{session_id}.json"
-
-    if not response_file.exists():
-        return None
-
-    try:
-        with open(response_file, 'r') as f:
-            response = json.load(f)
-
-        # Validate response has required field
-        if 'action' not in response:
-            log_debug(f"Response file missing 'action' field")
-            return None
-
-        # Delete file after reading (consume once)
-        response_file.unlink()
-        log_debug(f"✓ Read and deleted response file for session {session_id}")
-
-        return response
-    except json.JSONDecodeError as e:
-        log_debug(f"Invalid JSON in response file: {e}")
-        return None
-    except Exception as e:
-        log_debug(f"Error reading response file: {e}")
-        return None
-
-
 def get_auth_token() -> str | None:
     """Read auth token from Keychain or file."""
     # Try Keychain first - use same service name as CLI (com.watchcode.cli)
@@ -1069,11 +1030,45 @@ def get_auth_token() -> str | None:
     return None
 
 
+def decrypt_response(response_data: dict, auth_token: str) -> dict:
+    """Decrypt an E2E encrypted response from Watch.
+
+    If response is encrypted (has v, nonce, ciphertext), decrypts it.
+    If plaintext, returns as-is for backward compatibility.
+
+    Args:
+        response_data: Response dict from relay server
+        auth_token: Auth token for decryption
+
+    Returns:
+        Decrypted response dict
+    """
+    if not CRYPTO_AVAILABLE:
+        log_debug("[Crypto] cryptography module not available, returning plaintext")
+        return response_data
+
+    # Check if response is encrypted
+    if not is_encrypted(response_data):
+        log_debug("[Crypto] Response is plaintext (not encrypted)")
+        return response_data
+
+    try:
+        crypto = _get_crypto_manager()
+        decrypted = crypto.decrypt(response_data, auth_token)
+        log_debug(f"✓ [Crypto] Decrypted E2E response: action={decrypted.get('action')}")
+        return decrypted
+    except Exception as e:
+        log_debug(f"⚠️ [Crypto] Decryption failed: {e}, returning raw response")
+        # Return raw response - might be partially usable or a format issue
+        return response_data
+
+
 def check_relay_response(notification_id: str) -> dict | None:
     """Check relay server for Watch response.
 
     The Watch sends responses to POST /respond on the relay.
     The Mac polls GET /response/:auth_token/:notification_id to get them.
+    Responses may be E2E encrypted - decryption is handled automatically.
 
     Returns response dict if found, None otherwise.
     """
@@ -1094,35 +1089,57 @@ def check_relay_response(notification_id: str) -> dict | None:
     url = f"{RELAY_URL}/response/{auth_token}/{notification_id}"
 
     try:
-        import urllib.request
-        import urllib.error
-
-        req = urllib.request.Request(url, method='GET')
-        req.add_header('Accept', 'application/json')
-
-        with urllib.request.urlopen(req, timeout=5) as resp:
-            if resp.status == 200:
-                data = json.loads(resp.read().decode())
+        # LATENCY OPTIMIZATION: Use httpx with HTTP/2 if available
+        if HTTPX_AVAILABLE:
+            client = _get_relay_client()
+            resp = client.get(url, headers={'Accept': 'application/json'})
+            if resp.status_code == 200:
+                data = resp.json()
                 log_debug(f"[Relay Poll] Response: found={data.get('found')}")
-                # Response structure: { found: true, response: { action, timestamp, selected_options, ... } }
                 if data.get('found'):
                     response_data = data.get('response', {})
+
+                    # Decrypt if encrypted (E2E encryption)
+                    response_data = decrypt_response(response_data, auth_token)
+
                     action = response_data.get('action')
                     if action:
                         selected_opts = response_data.get('selected_options', [])
                         stored_notif_id = response_data.get('notification_id', 'unknown')
                         log_debug(f"✓ Relay response found: action={action}, selected={selected_opts}")
                         log_debug(f"  stored_notification_id={stored_notif_id}")
-                        # Return full response data including selected_options
                         return response_data
                 else:
-                    # Response not found - log for debugging
                     log_debug(f"[Relay Poll] No response stored for this notification_id")
-    except urllib.error.HTTPError as e:
-        if e.code != 404:  # 404 just means no response yet
-            log_debug(f"Relay HTTP error: {e.code}")
+            elif resp.status_code != 404:
+                log_debug(f"Relay HTTP error: {resp.status_code}")
         else:
-            log_debug(f"[Relay Poll] 404 - endpoint not found or no response")
+            # Fallback to urllib if httpx not available
+            import urllib.request
+            import urllib.error
+
+            req = urllib.request.Request(url, method='GET')
+            req.add_header('Accept', 'application/json')
+
+            with urllib.request.urlopen(req, timeout=5) as resp:
+                if resp.status == 200:
+                    data = json.loads(resp.read().decode())
+                    log_debug(f"[Relay Poll] Response: found={data.get('found')}")
+                    if data.get('found'):
+                        response_data = data.get('response', {})
+
+                        # Decrypt if encrypted (E2E encryption)
+                        response_data = decrypt_response(response_data, auth_token)
+
+                        action = response_data.get('action')
+                        if action:
+                            selected_opts = response_data.get('selected_options', [])
+                            stored_notif_id = response_data.get('notification_id', 'unknown')
+                            log_debug(f"✓ Relay response found: action={action}, selected={selected_opts}")
+                            log_debug(f"  stored_notification_id={stored_notif_id}")
+                            return response_data
+                    else:
+                        log_debug(f"[Relay Poll] No response stored for this notification_id")
     except Exception as e:
         log_debug(f"Relay poll error: {e}")
 
@@ -1181,224 +1198,51 @@ def send_invalidation(notification_id: str, reason: str = 'terminal_response') -
         return False
 
 
-def delete_stale_cloudkit_record(session_id: str) -> None:
-    """Delete a stale CloudKit record to prevent future false positives."""
-    if not CLOUDKIT_POLLER.exists():
-        return
-
-    # Use cloudkit_poller with --delete flag (we'll add this)
-    # For now, just run the poller once more which will trigger its cleanup
-    cmd = [
-        sys.executable,
-        str(CLOUDKIT_POLLER),
-        '--session-id', session_id,
-        '--delete-only'  # Delete without returning data
-    ]
-
-    try:
-        safe_env = os.environ.copy()
-        for var in ['LD_PRELOAD', 'LD_LIBRARY_PATH', 'DYLD_INSERT_LIBRARIES',
-                    'BASH_ENV', 'CDPATH', 'ENV']:
-            safe_env.pop(var, None)
-        safe_env['PATH'] = '/opt/homebrew/bin:/usr/local/bin:/usr/bin:/bin'
-
-        subprocess.run(
-            cmd,
-            capture_output=True,
-            text=True,
-            timeout=5,
-            env=safe_env
-        )
-        log_debug(f"Deleted stale CloudKit record for session {session_id}")
-    except Exception as e:
-        log_debug(f"Failed to delete stale record: {e}")
-
-
-def check_cloudkit_response(session_id: str, notification_id: str = '') -> dict | None:
-    """Check CloudKit for response from Watch (single check, no polling).
-
-    Watch writes directly to CloudKit, bypassing iPhone entirely.
-    This is more reliable than iCloud Drive for Watch responses.
-
-    If notification_id is provided, looks up by notification_id (unique per request).
-    Otherwise falls back to session_id (legacy behavior).
-
-    Returns response dict if found and valid, None otherwise.
-    """
-    if not CLOUDKIT_POLLER.exists():
-        # CloudKit poller not set up - skip silently
-        return None
-
-    # Prefer notification_id (unique per request) over session_id (shared across requests)
-    lookup_id = notification_id if notification_id else session_id
-
-    cmd = [
-        sys.executable,
-        str(CLOUDKIT_POLLER),
-        '--session-id', lookup_id,
-        '--once'  # Single check, no polling
-    ]
-
-    try:
-        safe_env = os.environ.copy()
-        for var in ['LD_PRELOAD', 'LD_LIBRARY_PATH', 'DYLD_INSERT_LIBRARIES',
-                    'BASH_ENV', 'CDPATH', 'ENV']:
-            safe_env.pop(var, None)
-        safe_env['PATH'] = '/opt/homebrew/bin:/usr/local/bin:/usr/bin:/bin'
-
-        result = subprocess.run(
-            cmd,
-            capture_output=True,
-            text=True,
-            timeout=5,
-            env=safe_env
-        )
-
-        if result.returncode == 0 and result.stdout.strip():
-            try:
-                response = json.loads(result.stdout)
-
-                # Validate timestamp - reject records older than 2 minutes
-                # This prevents stale records from being reused
-                record_timestamp = response.get('timestamp', 0)
-                if record_timestamp:
-                    # CloudKit timestamps are in milliseconds
-                    record_age_seconds = (time.time() * 1000 - record_timestamp) / 1000
-                    if record_age_seconds > 30:  # 30 seconds - permission requests are frequent
-                        log_debug(f"⚠️ CloudKit response too old ({record_age_seconds:.0f}s), deleting stale record")
-                        # Delete the stale record to prevent future false positives
-                        delete_stale_cloudkit_record(lookup_id)
-                        return None
-                    log_debug(f"✓ CloudKit response found for {lookup_id} (age: {record_age_seconds:.1f}s)")
-                else:
-                    log_debug(f"✓ CloudKit response found for {lookup_id} (no timestamp)")
-
-                return response
-            except json.JSONDecodeError as e:
-                log_debug(f"Invalid CloudKit response JSON: {e}")
-                return None
-
-    except subprocess.TimeoutExpired:
-        log_debug("CloudKit check timed out")
-    except Exception as e:
-        log_debug(f"CloudKit check error: {e}")
-
-    return None
-
-
-def send_wake_push(session_id: str, sequence_id: int, attempt: int) -> bool:
-    """Send silent wake push to wake iOS app.
-
-    This sends a background APNs notification that wakes the iPhone app
-    to process any queued WatchConnectivity messages.
-    """
-    if not APNS_SENDER.exists():
-        log_debug("apns_sender.py not found for wake push")
-        return False
-
-    cmd = [
-        sys.executable,
-        str(APNS_SENDER),
-        '--event', 'wake',
-        '--message', '',  # Empty for wake push
-        '--session-id', session_id,
-        '--push-type', 'background',
-        '--sequence-id', str(sequence_id),
-    ]
-
-    try:
-        safe_env = os.environ.copy()
-        for var in ['LD_PRELOAD', 'LD_LIBRARY_PATH', 'DYLD_INSERT_LIBRARIES',
-                    'BASH_ENV', 'CDPATH', 'ENV']:
-            safe_env.pop(var, None)
-        safe_env['PATH'] = '/opt/homebrew/bin:/usr/local/bin:/usr/bin:/bin'
-
-        result = subprocess.run(
-            cmd,
-            capture_output=True,
-            text=True,
-            timeout=10,
-            env=safe_env
-        )
-
-        if result.returncode == 0:
-            log_debug(f"[seq={sequence_id}] Wake push #{attempt} sent successfully")
-            return True
-        else:
-            log_debug(f"[seq={sequence_id}] Wake push #{attempt} failed: {result.stderr[:100] if result.stderr else 'no error'}")
-            return False
-
-    except subprocess.TimeoutExpired:
-        log_debug(f"[seq={sequence_id}] Wake push #{attempt} timed out")
-        return False
-    except Exception as e:
-        log_debug(f"[seq={sequence_id}] Wake push #{attempt} error: {e}")
-        return False
-
-
-def wait_for_response_with_wake(
-    session_id: str,
+def wait_for_response(
     sequence_id: int,
     timeout: int = DEFAULT_PERMISSION_TIMEOUT,
-    wake_interval: int = WAKE_PUSH_INTERVAL,
     notification_id: str = ''
 ) -> dict | None:
-    """Poll for response while sending periodic wake pushes.
+    """Poll relay server for Watch response.
 
-    This improves reliability when the iPhone app is suspended - wake pushes
-    will wake the app to process queued WatchConnectivity messages from Watch.
+    RELAY-ONLY: CloudKit/iCloud polling removed 2026-01-02 (security fix).
 
     Args:
-        session_id: Claude Code session ID
         sequence_id: Unique sequence for this request (for correlation)
         timeout: Total timeout in seconds
-        wake_interval: Seconds between wake pushes
-        notification_id: Unique notification ID for CloudKit lookup (if provided)
+        notification_id: Unique notification ID for relay lookup
 
     Returns:
         Response dict if received, None on timeout
     """
     start_time = time.time()
-    wake_count = 0
     poll_count = 0
-    last_wake = 0.0
 
-    log_debug(f"[seq={sequence_id}] Starting response wait (timeout={timeout}s, wake_interval={wake_interval}s, notification_id={notification_id[:8] if notification_id else 'none'}...)")
+    log_debug(f"[seq={sequence_id}] Starting response wait (timeout={timeout}s, notification_id={notification_id[:8] if notification_id else 'none'}...)")
 
     while time.time() - start_time < timeout:
         poll_count += 1
         elapsed = time.time() - start_time
 
-        # Check CloudKit first (Watch writes directly, bypasses iPhone)
-        # Use notification_id if provided (unique per request), otherwise fall back to session_id
-        response = check_cloudkit_response(session_id, notification_id)
-        if response:
-            log_debug(f"[seq={sequence_id}] ✓ CloudKit response after {elapsed:.1f}s, {poll_count} polls")
-            return response
-
-        # Also check iCloud Drive (iPhone relay path)
-        response = check_icloud_response(session_id)
-        if response:
-            log_debug(f"[seq={sequence_id}] ✓ iCloud Drive response after {elapsed:.1f}s, {poll_count} polls, {wake_count} wakes")
-            return response
-
-        # Check relay server (Watch sends responses here via POST /respond)
+        # RELAY-ONLY: Poll relay server for Watch response
+        # CloudKit/iCloud removed 2026-01-02 (security: public DB exposed auth tokens)
         response = check_relay_response(notification_id)
         if response:
             log_debug(f"[seq={sequence_id}] ✓ Relay response after {elapsed:.1f}s, {poll_count} polls")
             return response
 
-        # Send wake push every wake_interval seconds (for iPhone relay path)
-        if elapsed - last_wake >= wake_interval:
-            wake_count += 1
-            log_debug(f"[seq={sequence_id}] Sending wake push #{wake_count} at {elapsed:.1f}s")
-            send_wake_push(session_id, sequence_id, wake_count)
-            last_wake = elapsed
+        # LATENCY OPTIMIZATION: Adaptive polling intervals
+        # - First 5s: 200ms (user likely responding quickly)
+        # - 5-15s: 500ms (moderate wait)
+        # - 15s+: 1000ms (user taking time, reduce overhead)
+        if elapsed < 5:
+            time.sleep(0.2)
+        elif elapsed < 15:
+            time.sleep(0.5)
+        else:
+            time.sleep(1.0)
 
-        # Short sleep between polls (1 second)
-        time.sleep(1)
-
-    log_debug(f"[seq={sequence_id}] ⏱️ Timeout after {timeout}s, {poll_count} polls, {wake_count} wakes")
+    log_debug(f"[seq={sequence_id}] ⏱️ Timeout after {timeout}s, {poll_count} polls")
     return None
 
 
@@ -1487,14 +1331,11 @@ def handle_permission_request(data: dict) -> int:
         log_debug(f"[seq={sequence_id}] Failed to send notification, falling back to terminal")
         return 0  # No output = use terminal prompt
 
-    # 2. Wait for Watch response with periodic wake pushes
-    # Wake pushes help when iPhone is suspended and can't receive Watch messages immediately
-    log_debug(f"[seq={sequence_id}] Waiting for Watch response with wake pushes...")
-    response = wait_for_response_with_wake(
-        session_id=session_id,
+    # 2. Wait for Watch response via relay server
+    log_debug(f"[seq={sequence_id}] Waiting for Watch response...")
+    response = wait_for_response(
         sequence_id=sequence_id,
         timeout=DEFAULT_PERMISSION_TIMEOUT,
-        wake_interval=WAKE_PUSH_INTERVAL,
         notification_id=notification_id
     )
 
@@ -1526,45 +1367,67 @@ def handle_permission_request(data: dict) -> int:
 
 
 def handle_ask_user_question(tool_input: dict, session_id: str, project_name: str) -> int:
-    """Handle AskUserQuestion tool - send question with options to Watch.
+    """Handle AskUserQuestion tool - send question(s) with options to Watch.
 
-    Extracts question and options from tool_input, sends to relay,
-    waits for Watch response, and outputs the answer.
+    Supports both single and multi-question flows (1-4 questions).
+    Extracts all questions and options from tool_input, sends to relay,
+    waits for Watch response, and navigates terminal for each answer.
     """
     questions = tool_input.get('questions', [])
     if not questions:
         log_debug("No questions found in AskUserQuestion input")
         return 0
 
-    # For now, handle first question (most common case)
-    first_q = questions[0]
-    question_text = first_q.get('question', 'Question')
-    options = first_q.get('options', [])
-    multi_select = first_q.get('multiSelect', False)
-
-    # Convert options to relay format
-    question_options = []
-    for opt in options:
-        question_options.append({
-            'id': opt.get('label', '').lower().replace(' ', '_'),
-            'label': opt.get('label', '?'),
-            'description': opt.get('description', '')
-        })
-
-    log_debug(f"Sending AskUserQuestion with {len(question_options)} options: {question_text[:50]}...")
-
     # Generate unique IDs for this request
     notification_id = str(uuid.uuid4())
     sequence_id = generate_sequence_id()
 
-    # Send directly to relay with options
-    success = send_question_to_relay(
-        message=question_text,
-        session_id=session_id,
-        question_options=question_options,
-        multi_select=multi_select,
-        notification_id=notification_id
-    )
+    # Convert ALL questions to relay format
+    relay_questions = []
+    for idx, q in enumerate(questions):
+        question_text = q.get('question', f'Question {idx + 1}')
+        options = q.get('options', [])
+        multi_select = q.get('multiSelect', False)
+        header = q.get('header', None)
+
+        # Convert options to relay format
+        relay_options = []
+        for opt in options:
+            relay_options.append({
+                'id': opt.get('label', '').lower().replace(' ', '_'),
+                'label': opt.get('label', '?'),
+                'description': opt.get('description', '')
+            })
+
+        relay_questions.append({
+            'id': f"q{idx}",
+            'question': question_text,
+            'header': header,
+            'options': relay_options,
+            'multiSelect': multi_select
+        })
+
+    is_multi_question = len(relay_questions) > 1
+    log_debug(f"AskUserQuestion: {len(relay_questions)} question(s), multi={is_multi_question}")
+
+    # Send to relay with questions array
+    if is_multi_question:
+        # Multi-question flow - send full questions array
+        success = send_questions_to_relay(
+            questions=relay_questions,
+            session_id=session_id,
+            notification_id=notification_id
+        )
+    else:
+        # Single question - use legacy format for backward compatibility
+        first_q = relay_questions[0]
+        success = send_question_to_relay(
+            message=first_q['question'],
+            session_id=session_id,
+            question_options=first_q['options'],
+            multi_select=first_q.get('multiSelect', False),
+            notification_id=notification_id
+        )
 
     if not success:
         log_debug(f"⚠️ Failed to send AskUserQuestion to Watch, falling back to terminal")
@@ -1572,23 +1435,31 @@ def handle_ask_user_question(tool_input: dict, session_id: str, project_name: st
 
     log_debug(f"✓ AskUserQuestion sent, waiting for Watch response...")
 
-    # Wait for Watch response (same as permission_request)
-    response = wait_for_response_with_wake(
-        session_id=session_id,
+    # Wait for Watch response via relay server
+    response = wait_for_response(
         sequence_id=sequence_id,
         timeout=DEFAULT_PERMISSION_TIMEOUT,
-        wake_interval=WAKE_PUSH_INTERVAL,
         notification_id=notification_id
     )
 
     if response:
         action = response.get('action', '')
+
+        # Handle multi-question response (has 'answers' array)
+        answers = response.get('answers', [])
+        if answers and is_multi_question:
+            log_debug(f"✓ Received multi-question answers: {len(answers)} answers")
+            navigate_multi_question_answers(answers, questions)
+            return 0
+
+        # Handle single-question response (backward compatible)
         selected_options = response.get('selected_options', [])
         log_debug(f"✓ Received Watch answer: action={action}, selected={selected_options}")
 
         # For selectOption action, map selected label to option number
         if action == 'selectOption' and selected_options:
             selected_label = selected_options[0]
+            options = questions[0].get('options', [])
 
             # Handle custom/voice input (Watch sends "__custom__" with message)
             if selected_label == '__custom__':
@@ -1638,6 +1509,138 @@ def handle_ask_user_question(tool_input: dict, session_id: str, project_name: st
     return 0
 
 
+def navigate_multi_question_answers(answers: list, questions: list) -> bool:
+    """Navigate terminal UI for each answer in a multi-question flow.
+
+    Claude Code's AskUserQuestion presents questions sequentially.
+    For each answer, we navigate using arrow keys to select the option.
+
+    Args:
+        answers: List of answer dicts with questionId, selectedLabels, customText
+        questions: Original questions list from tool_input
+
+    Returns:
+        True if all navigations succeeded, False if any failed.
+    """
+    log_debug(f"Navigating {len(answers)} multi-question answers...")
+
+    for answer in answers:
+        question_id = answer.get('questionId', '')
+        selected_labels = answer.get('selectedLabels', [])
+        custom_text = answer.get('customText', '')
+
+        # Find the matching question to get its options
+        question_idx = None
+        for idx, q in enumerate(questions):
+            # Match by index (q0, q1, etc) or by original id
+            if question_id == f"q{idx}" or question_id == q.get('id', ''):
+                question_idx = idx
+                break
+
+        if question_idx is None:
+            log_debug(f"⚠️ Could not find question for answer: {question_id}")
+            continue
+
+        question = questions[question_idx]
+        options = question.get('options', [])
+
+        log_debug(f"Processing answer for question {question_idx + 1}: selected={selected_labels}")
+
+        # Handle custom/voice input
+        if selected_labels == ['__custom__'] and custom_text:
+            log_debug(f"Voice input for Q{question_idx + 1}: '{custom_text}'")
+            if inject_voice_response(custom_text, options):
+                log_debug(f"✓ Voice navigation succeeded for Q{question_idx + 1}")
+            else:
+                log_debug(f"⚠️ Voice navigation failed for Q{question_idx + 1}")
+        elif selected_labels:
+            # For single-select, navigate to the selected option
+            selected_label = selected_labels[0]
+            option_number = None
+            for idx, opt in enumerate(options, 1):
+                if opt.get('label') == selected_label:
+                    option_number = idx
+                    break
+
+            if option_number:
+                log_debug(f"Navigating to option {option_number} for Q{question_idx + 1}")
+                if navigate_to_option(option_number):
+                    log_debug(f"✓ Selected option {option_number} for Q{question_idx + 1}")
+                else:
+                    log_debug(f"⚠️ Navigation failed for Q{question_idx + 1}")
+            else:
+                log_debug(f"⚠️ Could not map '{selected_label}' to option for Q{question_idx + 1}")
+        else:
+            log_debug(f"⚠️ No selection for Q{question_idx + 1}")
+
+        # Small delay between questions to let Claude process each answer
+        time.sleep(0.3)
+
+    log_debug(f"✓ Multi-question navigation complete")
+    return True
+
+
+def send_questions_to_relay(
+    questions: list,
+    session_id: str,
+    notification_id: str = ''
+) -> bool:
+    """Send multi-question notification directly to relay with questions array.
+
+    This is used for AskUserQuestion with multiple questions (2-4).
+    """
+    auth_token = get_auth_token()
+    if not auth_token:
+        log_debug("No auth token found for questions notification")
+        return False
+
+    if not notification_id:
+        notification_id = str(uuid.uuid4())
+
+    # Build message from first question for notification banner
+    first_q = questions[0] if questions else {}
+    message = first_q.get('question', 'Multiple questions')[:100]
+    if len(questions) > 1:
+        message = f"{message} (+{len(questions) - 1} more)"
+
+    payload = {
+        'auth_token': auth_token,
+        'event': 'question',
+        'message': message,
+        'session_id': session_id,
+        'requires_action': True,
+        'notification_id': notification_id,
+        'questions': questions  # Full questions array for multi-question flow
+    }
+
+    try:
+        import urllib.request
+        import urllib.error
+
+        data = json.dumps(payload).encode('utf-8')
+        req = urllib.request.Request(
+            f"{RELAY_URL}/notify",
+            data=data,
+            method='POST'
+        )
+        req.add_header('Content-Type', 'application/json')
+
+        with urllib.request.urlopen(req, timeout=10) as resp:
+            if resp.status == 200:
+                log_debug(f"✓ Multi-question notification sent to relay ({len(questions)} questions)")
+                return True
+            else:
+                log_debug(f"Relay returned status {resp.status}")
+                return False
+
+    except urllib.error.HTTPError as e:
+        log_debug(f"Relay HTTP error: {e.code} - {e.read().decode()[:100]}")
+        return False
+    except Exception as e:
+        log_debug(f"Failed to send questions to relay: {e}")
+        return False
+
+
 def send_question_to_relay(
     message: str,
     session_id: str,
